// ----------------------------------------------------------------------
// File: tetrisesque.pov
// Vers: 3.6
// Date: 04/11/2006
// Auth: H.A. Camp
//
// Desc: Test of boundary mapping using predefined object grids
//

#version 3.6;


// ----------------------------------------------------------------------
// Render Settings
// ----------------------------------------------------------------------

#declare   Squares = 0; // Toggles valid (red/blue) square indicators (Final Render: 0)
#declare     TGrid = 0; // Toggles TGrid info in #debug stream        (Final Render: 0)
#declare    Blocks = 1; // Toggles Tetris blocks on/off               (Final Render: 1)
#declare AreaLight = 1; // Toggles area lighting                      (Final Render: 1)
#declare FocalBlur = 1; // Toggles Focal Blur to Scene                (Final Render: 1)
#declare Radiosity = 1; // Toggles Radiosity                          (Final Render: 1)


// ----------------------------------------------------------------------
// Declarations and Definitions
// ----------------------------------------------------------------------

#declare     Rbox = 1.0;
#declare       RS = seed(5873212);
#declare      RSx = seed(1475852);
#declare blocknum = 1;
#declare     XMin = -35;
#declare     XMax =  45;
#declare     ZMin = -10;
#declare     ZMax =  70;

#declare     Grid = array[XMax-XMin+2][ZMax-ZMin+2];
#declare     Area = box{<XMin-0.5,-0.2,ZMin-0.5>,<XMax+0.5,-0.1,ZMax+0.5> texture {pigment {cells pigment_map{[0.0 rgbt <0.5,0.5,0.5,0.95>][1.0 rgbt <1.0,1.0,1.0,0.95>]} translate 32}}} 


// ----------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------

#include       "colors.inc"
#include         "rand.inc"
#include       "shapes.inc"
#include      "strings.inc"
#include "tetris_shape.inc"
#include  "tetrisesque.inc"
#include   "tetris_mat.inc"

// ----------------------------------------------------------------------
// Global Settings
// ----------------------------------------------------------------------

global_settings {
  assumed_gamma 1.0
#if (Radiosity)
  radiosity {
    pretrace_start 0.32           // start pretrace at this size
    pretrace_end   0.01           // end pretrace at this size
    count 535                      // higher -> higher quality (1..1600) [35]
    nearest_count 6               // higher -> higher quality (1..10) [5]
    error_bound 1.8               // higher -> smoother, less accurate [1.8]
    recursion_limit 4             // how much interreflections are calculated (1..5+) [3]
    low_error_factor .05           // reduce error_bound during last pretrace step
    gray_threshold 0.0            // increase for weakening colors (0..1) [0]
    minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
    brightness 1                  // brightness of radiosity effects (0..1) [1]
    adc_bailout 0.01/4
  }
#end
}


// ----------------------------------------------------------------------
// Camera, Light, and Background Definitions
// ----------------------------------------------------------------------

camera {
//  location  <0, 30, 0>
  location  <0, 10, -15>
  look_at   <0, 0,  0>
#if (FocalBlur)
  aperture 0.15           // [0...N] larger is narrower depth of field (blurrier)
  blur_samples 40        // number of rays per pixel for sampling
  focal_point <0,0,0>    // point that is in focus <X,Y,Z>
  confidence 0.995           // [0...<1] when to move on while sampling (smaller is less accurate)
  variance 1/400            // [0...1] how precise to calculate (smaller is more accurate)
#end
}

background {color rgb 0}

#if (!AreaLight)
  light_source { <-30, 30, -30>  color rgb <1, 1, 1>}
  light_source { < 30, 10, -60>  color rgb <1, 1, 1>/4}
#else
  light_source {
    <-30, 10, -30>
    color rgb 2.0
    area_light
    <8, 0, 0> <0, 0, 8>  8, 8
    adaptive 0
    jitter  circular  orient
  }

  light_source {
    <30, 10, -60>
    color rgb 1/2
    area_light
    <8, 0, 0> <0, 0, 8>  4, 4
    adaptive 0
    jitter  circular  orient
  }
#end


// ----------------------------------------------------------------------
// Initialize Tetris Grid
// ----------------------------------------------------------------------

// First Pass - Determine valid Block Area

#declare X=XMin;
#declare Z=ZMin;
#while (Z <= ZMax)
  #while (X <= XMax)
    #declare X2 = X-XMin+1;
    #declare Z2 = Z-ZMin+1;

    #switch (F2(X-0.5,-1,Z-0.5))
    #range (0, 0.6)
      #declare Grid[X2][Z2] = "..";
    #break
    #else
      #declare Grid[X2][Z2] = "oo";
    #end
    
    #declare X=X+1;
  #end
  #declare X=XMin;
  #declare Z=Z+1;
#end

// Second Pass - Determine Block to Insert

#declare X=XMin;
#declare Z=ZMin;
#while (Z <= ZMax)
  #while (X <= XMax)
    #declare X2 = X-XMin+1;
    #declare Z2 = Z-ZMin+1;
    #if (!strcmp(Grid[X2][Z2],"oo"))
      
      #if (Blocks)
        Place_Block()
      #end


    #end
    #declare X=X+1;
  #end
  #declare X=XMin;
  #declare Z=Z+1;
#end


// ----------------------------------------------------------------------
// Display Tetris Grid
// ----------------------------------------------------------------------

#if (TGrid)
  ShowTGrid()
#end


// ----------------------------------------------------------------------
// Scene Description
// ----------------------------------------------------------------------

//plane {  y, -1 texture{ pigment{ color rgb <0.7,0.5,0.3>}}}

plane {  y, -1 material{HM2} hollow}


#declare X = XMin;
#declare Z = ZMin;
#while (Z<= ZMax)
  #while (X<= XMax)

    #declare X2 = X-XMin+1;
    #declare Z2 = Z-ZMin+1;

    // Adds red/blue boxes outlining valid placement positions
    #if (Squares)
      #switch (F2(X-0.5,-1,Z-0.5))
      #range (0, 0.6)
        box {0,1 texture{pigment {function {F0(X-x,y,Z-z)} pigment_map{[0.0 rgbt <0,0,1,0.4>][1.0 rgbt <1,0,0,0.9>]}}} scale <Rbox, 0.001, Rbox> translate <X-0.5+(1-Rbox),-1,Z-0.5+(1-Rbox)> no_shadow}
      #break
      #else
        box {0,1 texture{pigment {function {F0(X-x,y,Z-z)} pigment_map{[0.0 rgbt <1,0,0,0.4>][1.0 rgbt <1,0,0,0.9>]}}} scale <Rbox, 0.001, Rbox> translate <X-0.5+(1-Rbox),-1,Z-0.5+(1-Rbox)> no_shadow}
      #break
      #end
    #end
    
    #declare X = X + 1;
  #end
    #declare X = XMin;
    #declare Z = Z + 1;
#end

object {Area texture{pigment {color rgbt <0,0,1,0.9>}} translate <0,-1.0,0>}
/*
object {
  xobj[10] translate -0.5 rotate 90*0*y translate 0.5 scale <1,0.6,1> translate <0-0.5,-1.0,-1-0.5> 
  texture {
    pigment {color rgb <1,0,0>}
    finish {
      ambient 0.0
      diffuse 0.86
      brilliance 0.5
      phong 0.95
      phong_size 140
      specular 0.15
      roughness 0.0005
    }
  }
}
*/